@@ -2,7 +2,7 @@
 The main flow file to process one fastq file with
 """
 
-from trackcluster.utils import myexe, is_bin_in, get_file_prefix,del_files,parmap, list2file, file2list
+from trackcluster.utils import myexe, is_bin_in, get_file_prefix,del_files,parmap, name2file, file2name, list2file, file2list
 from trackcluster.convert import sam_to_bigGenePred
 from trackcluster.batch import process_one_junction_corrected_try, process_one_subsample_try
 from trackcluster.tracklist import read_bigg, write_bigg, cat_bed, bigg_count_write_native, is_a_read
@@ -16,6 +16,7 @@ from trackcluster.post import flow_desc, flow_class4
 import logging
 import os
 import functools
+from collections import deque
 
 #third party lib
 from pysam import AlignmentFile
@@ -79,6 +80,27 @@ def flow_bamconvert(wkdir,bamfile,out,prefix,score=30):
     return out
 
 
+def flow_add_gene(wkdir, prefix, bigg_gff_file, bigg_nano_file, f1=0.01, f2=0.05):
+    os.chdir(wkdir)
+
+    ### get two parts, the gene part and the novel part
+    # the gene part
+    outfile = prefix + "inter.bed"
+    # write the outfile to disk
+    wrapper_bedtools_intersect2_select(bigg_nano_file, bigg_gff_file, outfile=outfile,
+                                       fraction_bed1=f1, fraction_bed2=f2)
+    read_gene = get_gendic_bedinter(outfile)
+    print("read number in genes:", len(read_gene))
+
+    bigg_nano = read_bigg(bigg_nano_file)
+    bigg_new = tracklist_add_gene(bigg_nano, read_gene)
+
+    # cleanup
+    del_files([outfile])
+
+    return bigg_new
+
+
 def flow_preparedir(wkdir, prefix, bigg_gff_file, bigg_nano_file, genename_file="gene.txt",
                     f1=0.01, f2=0.05):
     """
@@ -94,15 +116,7 @@ def flow_preparedir(wkdir, prefix, bigg_gff_file, bigg_nano_file, genename_file=
 
     ### get two parts, the gene part and the novel part
     # the gene part
-    outfile=prefix+"inter.bed"
-    # write the outfile to disk
-    wrapper_bedtools_intersect2_select(bigg_nano_file, bigg_gff_file, outfile=outfile,
-                                                 fraction_bed1=f1, fraction_bed2=f2)
-    read_gene=get_gendic_bedinter(outfile)
-    print("read number in genes:", len(read_gene))
-
-    bigg_nano = read_bigg(bigg_nano_file)
-    bigg_new=tracklist_add_gene(bigg_nano, read_gene)
+    bigg_new=flow_add_gene(wkdir, prefix, bigg_gff_file, bigg_nano_file, f1, f2)
 
     bigg_ref= read_bigg(bigg_gff_file)
 
@@ -134,10 +148,9 @@ def flow_preparedir(wkdir, prefix, bigg_gff_file, bigg_nano_file, genename_file=
     # write the genename to file
     genename_l=list(gene_nano.keys())
 
-    list2file(genename_l, genename_file)
+    name2file(genename_l, genename_file)
+
 
-    # cleanup
-    del_files([outfile])
 
     return genename_l
 
@@ -200,10 +213,13 @@ def flow_count(wkdir, prefix, nano_bed, isoform_bed, gff_bed):
         bigg.get_subread_from_str()
 
         coverage = bigg.coverage
-        line_prefix=[bigg.geneName, bigg.name, coverage]
+
+        #geneName will be something like gene1 or gene1||gene2||gene3 (fusion case)
+        # but the isoform is still one for the fusion genes
+        # duplicate a line for the isoform, with mutiple lines for gene
+        # use deque for left append
+
         line_count=[0]*len(groups)
-
-
         for read in bigg.subread:
             if is_a_read(read,refname_set):
                 try:
@@ -212,16 +228,20 @@ def flow_count(wkdir, prefix, nano_bed, isoform_bed, gff_bed):
                     line_count[pos]+=1
                 except KeyError:
                     pass # the ref isoform should be skipped
-
         # print g1,g2,g3
         gsum = float(sum(line_count))
         if gsum < 0.0001:
-            line_one=line_prefix+[0]*len(groups)
+            line_count=[0]*len(groups)
         else:
             # adjust the count of each isoform, considering the dups in subreads
-            line_count_adj=[x/gsum*coverage for x in line_count]
-            line_one=line_prefix+line_count_adj
-        expression_list.append(line_one)
+            line_count=[x/gsum*coverage for x in line_count]
+        # the mutiple name part
+        genename_l=bigg.geneName.split("||")
+        line_prefix=deque([bigg.name, coverage])
+        for genename in genename_l:
+            line_prefix.appendleft(genename)
+            line_prefix.extend(line_count)
+            expression_list.append(line_prefix)
 
     out=prefix+"_exp.csv"
     line_head = ["gene", "isoform", "coverageall"] + groups
@@ -249,7 +269,7 @@ def flow_key_clusterj(wkdir, genename_file, core=30, batchsize=2000):
     """
     os.chdir(wkdir)
 
-    gene_l=file2list(genename_file)
+    gene_l=file2name(genename_file)
     process_one=functools.partial(process_one_junction_corrected_try, batchsize=batchsize)
 
     print("###### Run junction cluster ######")
@@ -345,7 +365,7 @@ def flow_key_cluster(wkdir, genename_file, core=30, batchsize=2000, intronweight
     :return:
     """
     os.chdir(wkdir)
-    gene_l=file2list(genename_file)
+    gene_l=file2name(genename_file)
     process_one=functools.partial(process_one_subsample_try,
                                   batchsize=batchsize, intronweight=intronweight,
                                   cutoff1=cutoff1, cutoff2=cutoff2, scorecutoff=scorecutoff)
@@ -430,9 +450,141 @@ def flow_cluster_all_gene_novel(wkdir, prefix,nano_bed, gff_bed, core=30,f1=0.01
 ########################################################################################################
 ########################################################################################################
 
-def flow_annotation(wkdir, ):
-    pass
+def flow_desc_annotation(wkdir,isoform_bed, gff_bed, offset=10, prefix=None):
+    """
+    generate all files needed to draw the desc figure for isoform classcification
+    :param wkdir:
+    :param prefix:
+    :param isoform_bed:
+    :param gff_bed:
+    :return:
+    """
+    if prefix is None:
+        prefix=get_file_prefix(isoform_bed, sep=".")
 
+    os.chdir(wkdir)
+    bigg_isoform=read_bigg(isoform_bed)
+    bigg_ref=read_bigg(gff_bed)
+
+    gene2isoform, _ = group_bigg_by_gene(bigg_isoform)
+    # filter the names from reference
+    gene2ref, _=group_bigg_by_gene(bigg_ref)
+
+    # get class4 and desc  for further use
+    # desc: [bigg0.name, bigg_ref.name, bigg_ref.geneName, desc_to_text(miss_desc), desc_to_text(extra_desc)]
+    # class4: [bigg0.name, class4]
+    class4 = []
+    desc = []
+    for k in gene2isoform.keys():
+        #print(k)
+        nanos = gene2isoform[k]
+        refs = gene2ref[k]
+
+        for bigg in nanos:
+            try:
+                out_class4 = flow_class4(bigg, refs, offset=offset)
+                class4.append(out_class4)
+                out_desc = flow_desc(bigg, refs, offset=offset)
+                desc.append(out_desc)
+            except IndexError:
+                pass
+    # IO part
+    list2file(desc, filename=prefix+"_desc.txt", sep="\t")
+    list2file(class4, filename=prefix+"_class4.txt", sep="\t")
+
+    # use class4 to make 3 class (the remained all belog to desc)
+    fullmatch_more = set([x[0] for x in class4 if "all_matched>" in x[1]])
+    fullmatch_less = set([x[0] for x in class4 if "all_matched_<" in x[1]])
+    new_junction = set([x[0] for x in class4 if "new_junction" in x[1]])
+
+    # use desc to make 7 classes
+    extra3 = set([x[0] for x in desc if "3 primer extra" in x[4]])
+    extra5 = set([x[0] for x in desc if "5 primer extra" in x[4]])
+    intron_retention = set([x[0] for x in desc if "Intron retention" in x[3]])
+    exon_miss = set([x[0] for x in desc if "Exon miss" in x[3]])
+    exon_extra = set([x[0] for x in desc if "Exon extra" in x[4]])
+    miss3 = set([x[0] for x in desc if "3 primer miss" in x[3]])
+    miss5 = set([x[0] for x in desc if "5 primer miss" in x[3]])
+
+    # make the fusion class separately
+    fusion_d = {}
+    for bigg in bigg_isoform:
+        genename_l=bigg.geneName.split("||")
+        if len(genename_l)>1:
+            fusion_d[bigg.name]=genename_l
+    fusion=set(list(fusion_d.keys()))
+
+    # IO for fusion
+    with open(prefix+"_fusion.txt", "w") as fw:
+        for k, v in fusion_d.items():
+            v_str = ";".join(v)
+            fw.write(k + "\t" + v_str + "\n")
+
+    # define class12 (with 11 for novel, the rest 1 for refererence annotation)
+    key_s=[bigg.name for bigg in bigg_isoform]
+    reference=set([bigg.name for bigg in bigg_isoform if bigg.ttype=="isoform_anno"])
+    key_s=list(set(key_s))
+
+    df = {}
+    for k in key_s:
+        for name, used_set in zip(["new_junction", "5'missing", "3'missing",
+                                   "5'extra", "3'extra",
+                                   "intron_retention", "inner_miss_exon", "inner_extra_exon",
+                                   "fusion_gene", "full_matched<", "full_matched>=", "reference"]
+                , [new_junction, miss5, miss3,
+                   extra5, extra3,
+                   intron_retention, exon_miss, exon_extra,
+                   fusion, fullmatch_less, fullmatch_more, reference]):
+            if k in used_set:
+                df[k] = name
+    # IO for class12
+    with open(prefix+"_class12.txt", "w") as fw:
+        for k, v in df.items():
+            fw.write(k+"\t"+v)
+            fw.write("\n")
+
+
+def flow_files_output(bigg_read_file, bigg_isoform_file, bigg_ref_file, prefix=None):
+    """
+    output all files used for further plotting
+    :param bigg_read_file:
+    :param bigg_isoform_file:
+    :param bigg_ref_file:
+    :param prefix:
+    :return:
+    """
+    if prefix is None:
+        prefix=get_file_prefix(bigg_read_file, sep=".")
+
+    read=read_bigg(bigg_read_file)
+
+    ####### length file for read, used for isoform
+    with open(prefix+"_read_"+"_mapped_len.txt", "w") as fw:
+        for bigg in read:
+            bigg.get_exon()
+            fw.write(bigg.name + "\t" + str(bigg.exonlen) + "\t" + bigg.geneName + "\n")
+
+    ####### fusion part
+    fusion_d = {}
+    used = set()
+
+    for bigg in read:
+        try:
+            fusion_d[bigg.name].append(bigg.geneName)
+        except KeyError:
+            fusion_d[bigg.name] = []
+            fusion_d[bigg.name].append(bigg.geneName)
+
+    fusion_d_real = {}
+    for k, v in fusion_d.items():
+        v_s = list(set(v))
+        if len(v_s) > 1:
+            fusion_d_real[k] = v
+
+    with open(prefix+"_fusion.txt", "w") as fw:
+        for k, v in fusion_d_real.items():
+            v_str = ";".join(v)
+            fw.write(k + "\t" + v_str + "\n")
 
 ############
 def flow_map_convert_clusterj_count(wkdir, prefix, ref_fasta, fastq_l, nano_bed, gff_bed, core=30,
